<!--
 * @Author: Jerry-Rain
 * @Date: 2020-03-24 13:48:00
 * @LastEditors: Jerry-Rain
 * @LastEditTime: 2020-04-23 17:00:22
 * @Description: 新电量统计页面
 -->
<template>
  <div>
    <!-- 总电量显示 -->
    <!-- <div class="total-electric">
      <span>当前累计总用电量：{{ chartData.total }} 度</span>
    </div> -->
    <!-- 按钮组 -->
    <div class="button-group">
      <div class="btn-mask">
        <div
          v-for="(item, index) in typeList"
          :key="index"
          class="btn"
          @click="btnIndex(index)"
          :style="{
            color: '#162c54',
            background: index !== selcetedIndex ? '#dde0e1' : 'white'
          }"
        >
          {{ item }}
        </div>
      </div>
      <span class="total-electric" v-show="!isNoData">{{ textList[selcetedIndex] + '用电量: ' + this.totalType(selcetedIndex) + '度' }}</span>
    </div>
    <!-- 图表区 -->
    <div class="chart-bg">
      <gree-block class="chart-block" v-show="!isNoData">
        <div class="chart-canves" ref="day" v-show="selcetedIndex === 0" />
        <div class="chart-canves" ref="week" v-show="selcetedIndex === 1" />
        <div class="chart-canves" ref="month" v-show="selcetedIndex === 2" />
        <div class="chart-canves" ref="year" v-show="selcetedIndex === 3" />
        <span v-show="!isNoData" class="unit-txt">/{{ type[selcetedIndex] }}</span>
      </gree-block>
      <div v-show="isNoData" class="isNoData">
        <p class="no-data-text" v-text="'当前无用电信息'" />
      </div>
      <gree-row v-show="!isNoData" class="tip">该设备的用电量仅供参考</gree-row>
    </div>
    <!-- 清除电量按钮 -->
    <gree-button class="btn-cln-elc" type="primary" :disabled="isNoData" @click="cleanElc" block round>电量清零</gree-button>
  </div>
</template>

<script>
import echarts from 'echarts';
import { Header, Row, Col, Block, Divider, Dialog, Button } from 'gree-ui';
import { mapState, mapMutations, mapActions } from 'vuex';
import chartsConfig from '@/mixins/echarts';
import { types } from '@/store/types';
import compact from 'lodash/compact';
import { showToast, getGridConList, clearHistoricalPhotovoltaicPowerData } from '@PluginInterface';

export default {
  name: 'ElectricChart',
  components: {
    [Col.name]: Col,
    [Row.name]: Row,
    [Block.name]: Block,
    [Button.name]: Button,
    [Dialog.name]: Dialog,
    [Header.name]: Header,
    [Divider.name]: Divider
  },
  mixins: [chartsConfig],
  data() {
    return {
      DEBUG: false, // 测试
      typeList: ['日报', '月报', '年报', '历年'],
      textList: ['今日累计', '当月累计', '今年累计', '累计总'],
      selcetedIndex: 0,
      type: ['h', '日', '月', '年'],
      yearChart: '',
      monthChart: '',
      dayChart: '',
      weekChart: '',
      isNoData: true, // 无数据显示
      tooltip: {
        position: function positionSetting(point, params, dom, rect, size) {
          //其中point为当前鼠标的位置，size中有两个属性：viewSize和contentSize，分别为外层
          console.info('tooltip2 point', point);
          // 该tooltip在echart中的各个参数
          console.info('tooltip2 params', params);
          // tooltip对应的dom元素
          console.info('tooltip2 dom', dom);
          // 未定义
          console.info('tooltip2 rect', rect);
          // size.contentSize 为tooltip的宽高 => [宽,高]
          // size.viewSize 为根容器的宽高 => [宽,高]
          console.info('tooltip2 size', size);

          var x = point[0]; //
          var boxWidth = size.contentSize[0];

          // 外部容器的宽度
          const contentWidth = size.viewSize[0];

          // 计算tooltip处于边界时的偏移量
          const offSetRight = x + boxWidth / 2;
          const offSetLeft = x - boxWidth / 2;
          if (offSetRight > contentWidth) {
            x = x - (offSetRight - contentWidth) - boxWidth / 2;
          } else if (offSetLeft < 0) {
            x = x - boxWidth / 2 - offSetLeft;
          } else {
            x = x - boxWidth / 2 + 5;
          }
          return [x, '0%'];
        },
        // formatter: '该时段用电量：{c} 度',
        trigger: 'axis', // 数据项图形触发
        axisPointer: {
          lineStyle: {
            type: 'dashed',
            color: '#DDE7EB'
          }
        },
        backgroundColor: '#77a7e0',
        // padding: 3,
        show: true,
        textStyle: {
          fontFamily: 'FZLTH--GB1-4',
          fontSize: '36px',
          color: '#fff'
        }
      }
    };
  },
  computed: {
    ...mapState('control', {
      mac: state => state.mac,
      isClear: state => state.isClear,
      chartData: state => state.chartData,
      dayTotal: state => state.chartData.dayTotal,
      monthTotal: state => state.chartData.monthTotal || 0,
      yearTotal: state => state.chartData.yearTotal || 0,
      allTotal: state => state.chartData.allTotal || 0
    })
  },
  mounted() {
    if (this.DEBUG) {
      // 调试模式时进入页面手动赋值
      this.setChartData(this.debugData);
      this.dayChartSet();
      this.refreshChart(this.chartData.dayChart[0], this.chartData.dayChart[1], 'dayChart');
    } else {
      this.getDayData();
    }
    this.initCharts(); // 初始化图表
  },
  beforeDestroy() {
    this.setChartData({
      dayChart: null,
      weekChart: null,
      monthChart: null,
      yearChart: null
    });
  },
  methods: {
    ...mapMutations('control', {
      setChartData: 'SET_CHART_DATA'
    }),
    ...mapActions({
      sendCtrl: types.SEND_CTRL
    }),
    // 初始化图表
    initCharts() {
      this.dayChart = echarts.init(this.$refs.day);
      this.weekChart = echarts.init(this.$refs.week);
      this.monthChart = echarts.init(this.$refs.month);
      this.yearChart = echarts.init(this.$refs.year);
    },
    totalType(selcetedIndex) {
      const totalList = [this.dayTotal, this.monthTotal, this.yearTotal, this.allTotal];
      const result = totalList[selcetedIndex];
      return result;
    },
    // 当前选择类型请求对应的数据
    btnIndex(index) {
      this.DEBUG ? this.setChartData(this.debugData) : ''; // 测试数据
      this.selcetedIndex = index;

      this.otherTooltip = {
        position: function positionSetting(point, params, dom, rect, size) {
          //其中point为当前鼠标的位置，size中有两个属性：viewSize和contentSize，分别为外层
          var x = point[0]; //
          var boxWidth = size.contentSize[0];
          return [x - boxWidth / 2 + 5, '12%'];
        },
        // formatter: '该日用机时长：{c}  h',
        trigger: 'axis', // 数据项图形触发
        axisPointer: {
          lineStyle: {
            type: 'dashed'
          }
        },
        backgroundColor: '#77a7e0',
        // padding: 3,
        show: true,
        textStyle: {
          fontFamily: 'FZLTH--GB1-4',
          fontSize: '36px',
          color: '#fff'
        }
      };
      const dataZoom = [
        {
          show: false,
          start: 1,
          end: 50
        },
        {
          type: 'inside'
        }
      ];
      switch (index) {
        case 0:
          this.chartData.dayChart ? this.dayChartSet() : this.getDayData();
          break;
        case 1:
          this.chartData.weekChart ? this.weekChartSet(dataZoom) : this.getWeekData();
          break;
        case 2:
          this.chartData.monthChart ? this.monthChartSet(dataZoom) : this.getMonthData();
          break;
        case 3:
          this.chartData.yearChart ? this.yearChartSet(dataZoom) : this.getYearData();
          break;
        default:
          break;
      }
    },
    // 清除电量dialog
    cleanElc() {
      Dialog.confirm({
        title: '提示',
        message: '电量清零后不可恢复，是否确定清零？',
        confirmButtonText: '确定',
        cancelButtonText: '取消'
      })
        .then(() => {
          this.cleanElcInterface();
        })
        .catch(() => {
          console.log('cancel');
        });
    },
    getDayData() {
      getGridConList(this.mac, 'day')
        .then(res => {
          // hideLoading();
          const obj = JSON.parse(res);
          console.log('日用电量5656456468', obj);
          const { time, val } = obj.data;

          let obj1 = {};
          let realValue = 0;
          let realValueArr = [];
          let value = new Array(12).fill(0);
          let count = [];
          const arr1 = Object.values(val);
          const arr2 = Object.values(time);

          this.isNoData = !compact(val).length;
          //  日求和
          if (val.length > 0) {
            let result = val[val.length - 1];
            this.setChartData({ dayTotal: result.toFixed(3) });
          }
          arr1.forEach((v, i) => {
            realValue = v - arr1[i - 1];
            realValueArr.push(realValue);
            realValueArr[0] = ~~realValueArr[0];
          });

          for (let i = 0; i < time.length; i++) {
            const objKey = time[i];
            const objValue = realValueArr[i];
            obj1[objKey] = objValue;
          }
          arr2.forEach(v => {
            value[v / 2] = obj1[v].toFixed(3);
          });
          for (let i = 0; i < 12; i += 1) {
            count[i] = 2 * i + ':00';
          }
          this.setChartData({ dayChart: [value, count] });
          this.dayChartOpt.tooltip.formatter = '该时段用电量：{c} 度';
          this.refreshChart(value, count, 'dayChart');
        })
        .catch(res => {
          console.log('查询失败', res);
          showToast('查询失败----', 0);
        });
    },
    getWeekData() {
      getGridConList(this.mac, 'month')
        .then(res => {
          const obj = JSON.parse(res);
          const { time, val } = obj.data;

          const d = new Date(new Date().getFullYear(), new Date().getMonth() + 1, 0);
          const monthCount = d.getDate();
          const month = [];
          const count = [];
          //  月求和
          if (val.length > 0) {
            let result = val.reduce(function(prev, cur) {
              return prev + cur;
            });
            this.setChartData({ monthTotal: result.toFixed(3) });
          }
          for (let i = 0; i < monthCount; i += 1) {
            month[i] = 0;
            count[i] = i + 1;
          }

          for (let i = 0; i < time.length; i += 1) {
            month[time[i] - 1] = val[i];
          }

          this.isNoData = !compact(val).length;
          this.setChartData({ weekChart: [month, count] });
          this.tooltip.formatter = '当日用电量：{c} 度';
          this.otherOpt.tooltip = this.tooltip;
          this.refreshChart(month, count, 'weekChart');
        })
        .catch(res => {
          console.log('查询失败', res);
          showToast('查询失败', 0);
        });
    },
    getMonthData() {
      // showLoading();
      getGridConList(this.mac, 'year')
        .then(res => {
          // hideLoading();
          const obj = JSON.parse(res);
          const { time, val } = obj.data;

          const year = [];
          const count = [];

          //  年求和
          if (val.length > 0) {
            let result = val.reduce(function(prev, cur) {
              return prev + cur;
            });
            this.setChartData({ yearTotal: result.toFixed(3) });
          }
          for (let i = 0; i < 12; i += 1) {
            year[i] = 0;
            count[i] = i + 1;
          }

          for (let i = 0; i < time.length; i += 1) {
            year[time[i] - 1] = val[i].toFixed(3);
          }
          this.isNoData = !compact(val).length;
          this.setChartData({ monthChart: [year, count] });
          this.tooltip.formatter = '当月用电量：{c} 度';
          this.otherOpt.tooltip = this.tooltip;
          this.refreshChart(year, count, 'monthChart');
        })
        .catch(res => {
          console.log('查询失败', res);
          showToast('查询失败', 0);
          // hideLoading();
        });
    },
    getYearData() {
      // showLoading();
      getGridConList(this.mac, 'all')
        .then(res => {
          // hideLoading();
          const obj = JSON.parse(res);
          const { time, val } = obj.data;
          console.log('Ele------', obj);
          const all = [];
          const count = [];
          let TOTAL = 0;
          for (let i = 0; i < time.length; i += 1) {
            all[i] = val[i].toFixed(3);
            count[i] = time[i];
            TOTAL += val[i];
          }
          this.isNoData = !compact(val).length;
          this.setChartData({ yearChart: [all, count], allTotal: TOTAL.toFixed(3) });
          this.tooltip.formatter = '当年用电量：{c} 度';
          this.otherOpt.tooltip = this.tooltip;
          this.refreshChart(all, count, 'yearChart');
        })
        .catch(res => {
          console.log('查询失败', res);
          showToast('查询失败', 0);
          // hideLoading();
        });
    },
    // 清除电量接口
    cleanElcInterface() {
      clearHistoricalPhotovoltaicPowerData(this.mac)
        .then(result => {
          // hideLoading();
          const res = JSON.parse(result);
          // 判断是否成功清除电量
          if (res.r === 400) {
            console.log('Api调用失败');
          } else if (res.r === 200) {
            console.log('成功请求，清除电量');
            showToast('清除成功', 0);
            this.sendCtrl({ Electnumzero: 0 });
            this.chartsResize();
            this.setChartData({
              dayTotal: 0,
              monthTotal: 0,
              yearTotal: 0,
              allTotal: 0
            });
            window.$bus.$emit('claerData');
          } else if (res.r === 10700) {
            showToast('您没有权限清除电量哦', 0);
          } else {
            console.log('失败！其他原因');
          }
        })
        .catch(err => {
          console.log('err', err);
        });
    },
    chartsResize() {
      this.dayChart.clear();
      this.weekChart.clear();
      this.monthChart.clear();
      this.yearChart.clear();
      this.isNoData = true;
    },
    refreshChart(value, count, type) {
      this.otherOpt.xAxis.data = count;
      this.otherOpt.series[0].data = value;
      this.dayChartOpt.xAxis.data = count;
      this.dayChartOpt.series[0].data = value;
      setTimeout(() => {
        this[type].setOption(type === 'dayChart' ? this.dayChartOpt : this.otherOpt, true);
        this.$nextTick(() => {
          this[type].resize();
        });
      }, 0);
    },
    dayChartSet() {
      this.tooltip.formatter = '该时段用电量:{c} 度';
      this.dayChartOpt.tooltip = this.tooltip;
      this.dayChartOpt.grid.show = false;
      this.dayChartOpt.grid = { left: '2%', top: '15%', right: '2%', bottom: '10%', containLabel: false };
      this.refreshChart(this.chartData.dayChart[0], this.chartData.dayChart[1], 'dayChart');
    },
    weekChartSet(dataZoom) {
      this.tooltip.formatter = '当日用电量：{c} 度';
      this.otherOpt.tooltip = this.tooltip;
      this.otherOpt.dataZoom = dataZoom;
      this.otherOpt.grid = { left: '2%', top: '15%', right: '2.5%', bottom: '10%' };
      this.refreshChart(this.chartData.weekChart[0], this.chartData.weekChart[1], 'weekChart');
    },
    monthChartSet() {
      this.tooltip.formatter = '当月用电量：{c} 度';
      this.otherOpt.tooltip = this.tooltip;
      this.otherOpt.dataZoom = [];
      this.otherOpt.grid = { left: '2%', top: '15%', right: '2%', bottom: '10%' };
      this.refreshChart(this.chartData.monthChart[0], this.chartData.monthChart[1], 'monthChart');
    },
    yearChartSet() {
      this.tooltip.formatter = '当年用电量：{c} 度';
      this.otherOpt.tooltip = this.tooltip;
      this.otherOpt.dataZoom = [];
      this.otherOpt.grid = { left: '2%', top: '15%', right: '2%', bottom: '10%' };
      this.refreshChart(this.chartData.yearChart[0], this.chartData.yearChart[1], 'yearChart');
    },
    goBack() {
      this.$router.go(-1);
    }
  }
};
</script>
